;;; permas_dat.el --- PERMAS DAT mode (editing PERMAS data files)

;; Copyright (C) 1999-2015 Free Software Foundation, Inc.

;; Last Change: 19-Jun-18
;; Keywords: PERMAS, data, editing

;; This file is part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; permas-dat-mode is a major mode for editing PERMAS dat files.

;;; Code:
(require 'rainbow-mode)
(setq rainbow-hexadecimal-colors-font-lock-keywords
      (append rainbow-hexadecimal-colors-font-lock-keywords
 '(("#[0-9a-fA-F][0-9a-fA-F]\\(\\([0-9a-fA-F]\\{6\\}\\)\\)"
      (0 (rainbow-colorize-hexadecimal-without-sharp))))))

;;;###autoload
(defun permas-dat-mode ()
  "Text-mode for viewing/editing PERMAS DAT-Files."
  (interactive) (text-mode) (setq mode-name  "PERMAS-DAT")
  (setq major-mode 'permas-dat-mode)
;;
  (make-local-variable 'comment-start) (setq comment-start "!")
  (make-local-variable 'kf-def-regexp) (setq kf-def-regexp "^[^!]*[$]")
;;
  (use-local-map permas-dat-mode-map)
  (local-set-key "\e\C-a"  'permas-dat-prev)
  (local-set-key "\e\C-e"  'permas-dat-next)
  (substitute-key-definition 'split-line 'dat-split-line (current-local-map) global-map)
;;
; (setq context-help-function   'PERMAS-dat-help)
; (setq context-update-function 'PERMAS-dat-context-update)
  (setq compile-buffer-function 'PERMAS-compile-buffer)
  (set (make-local-variable 'font-lock-defaults) '(permas-dat-font-lock-keywords t t))
  (if font-lock-mode (font-lock-mode 1)) ;no guess why we need this
  (if font-lock-mode (rainbow-turn-on))  ;colorize PART colors
  (run-mode-hooks 'permas-dat-mode-hook))

(defvar permas-dat-mode-map (make-sparse-keymap)
   "Local keymap used in permas-dat-mode.")
;;
(defun dat-split-line ()
  "Break line at point and insert continuation marker and alignment."
  (interactive) (delete-horizontal-space) (let (colpos)
  (if (save-excursion (re-search-backward "^\\|!")
                      (setq colpos (current-column)) (looking-at "!"))
      (insert (concat "\n" (make-string colpos ? ) "! "))
      (if (looking-at " *!") (insert "\n") (insert "\n      & ")))))

;;;###autoload
(defun permas-cas-mode ()
  "Text-mode for viewing/editing PERMAS CAS[O]-Files."
  (interactive) (permas-dat-mode)
  (setq mode-name "PERMAS-CAS-File") (setq major-mode 'permas-cas-mode)
  (run-mode-hooks 'permas-cas-mode-hook))

;;;###autoload
(defun permas-caso-mode ()
  "Text-mode for viewing PERMAS CAS[O]-Files."
  (interactive) (permas-dat-mode)
  (setq mode-name "PERMAS-CASO-File") (setq major-mode 'permas-caso-mode)
  (run-mode-hooks 'permas-caso-mode-hook))



;; font-lock support (list of regular expressions for highlighting)
;;-----------------------------------------------------------------
(setq permas-dat-font-lock-keywords
  (list
   ;; $ENTER/EXIT COMPONENT/MATERIAL
   '("\$ *\\(enter\\|exit\\)[:blanks:]*[^ \t\n]*" . 'font-lock-match-face)
   ;;
   ;; $VARIANTNAME and $END VARIANTNAME
   '("\$ *\\(end\\)? *\\(situation\\|structure\\|system\\|constraints\\|loading\\|modification\\|results\\|material\\|laminate\\|sandwich\\|fluid\\|flfsolid\\|gasket\\)"
                                       . 'font-lock-separator-face)
   ;;
   ;; $KEYWORD subkeys (general header-keywords)
   '("\$[a-z0-9_ \t]+"                 . 'font-lock-function-name-face)
   ;;
   ;; keyword = value
   '("=\\([^;:=!\n]*\\)"              (1 'font-lock-variable-name-face t))
   '(" \\([a-z][^ ]*\\) *="           (1 'font-lock-type-face t)) ; t=Overwrite
   ;;
   ;; continuation-fields (and following list of integers/reals)
   '("[&] +[ 0-9]*$"                   . 'font-lock-string-face)
   '("[&]"                             . 'font-lock-string-face)
   ;;
   ;; data-field separator
   '("[;:]"                            . 'font-lock-separator-face)
   ;;
   ;; data-name (e.g. leading setname, contname, geodatname on data-lines)
   '("^ *\\([a-z][^ ]*\\)\\b"         (1 'font-lock-variable-name-face))
   '("^ *\\(#[0-9]+\\)"               (1 'font-lock-variable-name-face))
   ;;
   ;; Open list of IDs
   '("^ *\\([0-9]+\\) \\([^!\n]+\\)"  (2 'font-lock-string-face))
   '(":\\([ 0-9]+\\)[!\n]"            (1 'font-lock-string-face))
   ;;
   ;; Trailing numbers
   '("& *:\\([ 0-9.ed+-]*[!\n]\\)"    (1 'font-lock-string-face))
   ;;
   ;; Standard comments
   '("!.*"                            (0 'font-lock-comment-face t))
))


;; Next/prev header line
;;-----------------------
(defun permas-dat-next (&optional arg)
  "Move forward to next data block. With argument, do it that many times.
Negative argument -N means move back to Nth preceding end of xydata."
  (interactive "p") (if (or (null arg) (= arg 0)) (setq arg 1))
  (forward-char 1) (permas-dat-prev (- arg)))
;;
(defun permas-dat-prev (&optional arg)
"Move to start of current data block. With argument, do it that many times.
Negative arg -N means move forward to Nth following start of data block."
  (interactive "p") (or (bolp) (end-of-line 1))
  (and (re-search-backward "^[ \t]*[$]" nil 'move (or arg 1))
       (progn (beginning-of-line)
          (if (looking-at "^[ \t]*[$]END")
              (if (> arg 0) (permas-dat-prev) (permas-dat-next)))
          (if (looking-at "^[ \t]*[$]EXIT")
              (if (> arg 0) (permas-dat-prev) (permas-dat-next)))
        t)))

;;;----------------------------------------------------------------------------
;(defun PERMAS-dat-help (&optional query)
;"Show help for PERMAS DAT input command of actual line (info from um450).
;Ask interactively for keyword if QUERY is non-nil (short cut otherwise)."
; (interactive)
; (let ((key (keyword-at-point)) (uci))
;      (if query (let ((enable-recursive-minibuffers t))
;                (setq key (read-with-default "PERMAS input command" key))))
;      (help-uci-key key)
;))
;;;----------------------------------------------------------------------------

;;
(provide 'permas_dat)

;;; permas_dat.el ends here
